precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float iGlobalTime;
uniform vec2 iResolution;

float d;

vec3 rgb2hsb(vec3 c ){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz),
                 vec4(c.gb, K.xy),
                 step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r),
                 vec4(c.r, p.yzx),
                 step(p.x, c.r));
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)),
                d / (q.x + e),
                q.x);
}

vec3 hsb2rgb(vec3 c ){
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0,4.0,2.0),
                             6.0)-3.0)-1.0,
                     0.0,
                     1.0 );
    rgb = rgb*rgb*(3.0-2.0*rgb);
    return c.z * mix(vec3(1.0), rgb, c.y);
}

float lookup(vec2 p, float dx, float dy)
{
    vec2 uv = p + vec2(dx * d, dy * d) / iResolution.xy;
    vec4 c = texture2D(inputImageTexture, uv.xy);

	// return as luma
    return 0.2126*c.r + 0.7152*c.g + 0.0722*c.b;
}

vec4 blendHardLight(vec4 blend, vec4 base){
    return vec4(base.r < 0.5 ? (2.0 * base.r * blend.r) : (1.0 - 2.0 * (1.0 - base.r) * (1.0 - blend.r)),
                base.g < 0.5 ? (2.0 * base.g * blend.g) : (1.0 - 2.0 * (1.0 - base.g) * (1.0 - blend.g)),
                base.b < 0.5 ? (2.0 * base.b * blend.b) : (1.0 - 2.0 * (1.0 - base.b) * (1.0 - blend.b)),
                1.0);
}

void main()
{
    d = 4.; // kernel offset
    vec2 uv = textureCoordinate;

	// simple sobel edge detection
    float gx = 0.0;
    gx += -1.0 * lookup(uv, -1.0, -1.0);
    gx += -2.0 * lookup(uv, -1.0,  0.0);
    gx += -1.0 * lookup(uv, -1.0,  1.0);
    gx +=  1.0 * lookup(uv,  1.0, -1.0);
    gx +=  2.0 * lookup(uv,  1.0,  0.0);
    gx +=  1.0 * lookup(uv,  1.0,  1.0);

    float gy = 0.0;
    gy += -1.0 * lookup(uv, -1.0, -1.0);
    gy += -2.0 * lookup(uv,  0.0, -1.0);
    gy += -1.0 * lookup(uv,  1.0, -1.0);
    gy +=  1.0 * lookup(uv, -1.0,  1.0);
    gy +=  2.0 * lookup(uv,  0.0,  1.0);
    gy +=  1.0 * lookup(uv,  1.0,  1.0);

    float g = sqrt(gx*gx + gy*gy);

    vec4 col = texture2D(inputImageTexture, uv);

    vec4 color1 = blendHardLight(col, col * g);
    vec3 hsb1 = rgb2hsb(color1.rgb);
    hsb1.g += 0.5;

    vec4 color2 = vec4(hsb2rgb(hsb1), 1.0);
    color2 += vec4(0.2, -0.05, -0.1, 0.0);//vec4(0.078, -0.019, -0.039, 0.0);
    vec3 hsb2 = rgb2hsb(color2.rgb);

    vec4 color3 = vec4(hsb2rgb(vec3(hsb2.r, hsb2.g, hsb1.b)), 1.0);

    gl_FragColor = color3;
}


