precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float iGlobalTime;
uniform vec2 iResolution;

float d;

vec2 center = vec2(0.5,0.5);
float speed = 0.4;

vec3 bg(vec2 uv)
{
    float invAr = 16. / 9.;

	vec3 col = vec4(.0, .5 ,0.5,1.0).xyz;

     vec3 texcol;

	float x = (center.x-uv.x);
	float y = (center.y-uv.y) *invAr;

	float r = -sqrt(x*x + y*y); //uncoment this line to symmetric ripples
	float z = 1.0 + 0.8*sin((r+iGlobalTime*speed)/0.06);

	texcol.x = z;
	texcol.y = z;
	texcol.z = z;

	return col*texcol;
}

float lookup(vec2 p, float dx, float dy)
{
    vec2 uv = p + vec2(dx * d, dy * d) / iResolution.xy;
    vec4 c = texture2D(inputImageTexture, uv.xy);

	// return as luma
    return 0.2126*c.r + 0.7152*c.g + 0.0722*c.b;
}

void main()
{
    d = 2.; // kernel offset
    vec2 uv = textureCoordinate;

	// simple sobel edge detection
    float gx = 0.0;
    gx += -1.0 * lookup(uv, -1.0, -1.0);
    gx += -2.0 * lookup(uv, -1.0,  0.0);
    gx += -1.0 * lookup(uv, -1.0,  1.0);
    gx +=  1.0 * lookup(uv,  1.0, -1.0);
    gx +=  2.0 * lookup(uv,  1.0,  0.0);
    gx +=  1.0 * lookup(uv,  1.0,  1.0);

    float gy = 0.0;
    gy += -1.0 * lookup(uv, -1.0, -1.0);
    gy += -2.0 * lookup(uv,  0.0, -1.0);
    gy += -1.0 * lookup(uv,  1.0, -1.0);
    gy +=  1.0 * lookup(uv, -1.0,  1.0);
    gy +=  2.0 * lookup(uv,  0.0,  1.0);
    gy +=  1.0 * lookup(uv,  1.0,  1.0);

    float g = gx*gx + gy*gy;

    vec4 col = texture2D(inputImageTexture, uv);

    vec4 bgColor = vec4(bg(uv), 1.);

    gl_FragColor = col + bgColor * g;
}


