precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float iGlobalTime;
const float STRENGTH = 0.6;
const float PI = 3.141592653589793;

float Linear_ease(in float begin, in float change, in float duration, in float time) {
    return change * time / duration + begin;
}

float Exponential_easeInOut(in float begin, in float change, in float duration, in float time) {
    if (time == 0.0)
        return begin;
    else if (time == duration)
        return begin + change;
    time = time / (duration / 2.0);
    if (time < 1.0)
        return change / 2.0 * pow(2.0, 10.0 * (time - 1.0)) + begin;
    return change / 2.0 * (-pow(2.0, -10.0 * (time - 1.0)) + 2.0) + begin;
}

float Sinusoidal_easeInOut(in float begin, in float change, in float duration, in float time) {
    return -change / 2.0 * (cos(PI * time / duration) - 1.0) + begin;
}

float random(in vec3 scale, in float seed) {
     return fract(
                    sin(
                        iGlobalTime
                        ) * 43758.5453
                 );
}

vec3 crossFade(in vec2 uv) {
    return texture2D(inputImageTexture, uv).rgb;
}

vec4 getMask(vec2 uv)
{
    vec4 color3 = vec4(1., 1., 1., 1.);
    vec4 color2 = vec4(.64, .9, 1., 1.);
    vec4 color1 = vec4(.76, .58, 1., 1.);
    vec4 retColor;
    if(uv.y<.5){
        retColor =  mix(color1, color2, uv.y * 2.);
    } else {
        retColor =  mix(color2, color3, (uv.y - .5)*2.);
    }

    return retColor;
}

float psAdd(float a, float b){
    if(a<=.5){
        return a * b * 2.;
    } else {
        return 1. - 2. *(1. -a)*(1.-b);
    }
}

void main() {
    vec2 texCoord = textureCoordinate;
	float progress = sin(iGlobalTime * 2.) * 0.5 + 0.5;
    // Linear interpolate center across center half of the image
    vec2 center = vec2(Linear_ease(0.5, 0.0, 1.0, progress),0.5);

    // Mirrored sinusoidal loop. 0->strength then strength->0
    float strength = Sinusoidal_easeInOut(0.0, STRENGTH, 0.5, progress);

    vec3 color = vec3(0.0);
    float total = 0.0;
    vec2 toCenter = center - texCoord;

    /* randomize the lookup values to hide the fixed number of samples */
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0)*0.5;

    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 1.0 * (percent - percent * percent);
        color += crossFade(texCoord + toCenter * percent * strength) * weight;
        total += weight;
    }

    vec4 oriColor = vec4(color / total, 1.);
    vec4 maskColor = getMask(texCoord);
    float ratio = strength * .75;
    vec4 colorRet = vec4(psAdd(oriColor.r, maskColor.r),
                            psAdd(oriColor.g, maskColor.g),
                            psAdd(oriColor.b, maskColor.b),
                            1.);
    gl_FragColor = mix(oriColor, colorRet, ratio);
}